import "Thurallor.FriendAlert";

local hello = L:GetText("/HelloMessage");
hello = string.gsub(hello, "<version>", plugin:GetVersion());
Turbine.Shell.WriteLine(hello);
_G.PutsPrefix = "<rgb=#DAA520>Friend Alert:</rgb> ";
FriendLoginPattern = L:GetText("/ChatLog/FriendLoginPattern");
FriendLogoutPattern = L:GetText("/ChatLog/FriendLogoutPattern");
KinshipLoginPattern = L:GetText("/ChatLog/KinshipLoginPattern");
KinshipLogoutPattern = L:GetText("/ChatLog/KinshipLogoutPattern");

Window = Turbine.UI.Window();
Window:SetVisible(true);
Window:SetMouseVisible(false);
Window:SetSize(Turbine.UI.Display.GetWidth(), Turbine.UI.Display.GetHeight());
Window:SetZOrder(0x7fffffff);
Label = Turbine.UI.Label();
Label:SetParent(Window);
Label:SetMouseVisible(false);
Label:SetFont(Turbine.UI.Lotro.Font.TrajanProBold36);
Label:SetFontStyle(Turbine.UI.FontStyle.Outline);
Label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
Label:SetMultiline(true);
Label:SetSize(Window:GetSize());

Messages = {};     -- Message display queue
NewMessages = {};  -- Recently received messages
HarvestDelay = 1;  -- Maximum amount of time between a friend login being
                   -- reported in the Standard channel and the same login
                   -- being reported in the Kinship channel

function Window.Update()
    local currentTime = Turbine.Engine.GetGameTime();
    if (currentTime >= NextUpdateTime) then
        -- Move all new messages into the display queue
        for _, args in pairs(NewMessages) do
            table.insert(Messages, { args.message, args.color });
            Window:SetVisible(true);
        end
        NewMessages = {};
    end

    if (next(Messages) and not FadeEndTime) then
        -- Display the next message and start the fade timer
        FadeEndTime = currentTime + Settings.displayTime + Settings.fadeTime;
        local msg, color = unpack(table.remove(Messages, 1));
        Label:SetForeColor(color);
        Label:SetText(msg);
    elseif (FadeEndTime) then
        local progress = (currentTime - (FadeEndTime - Settings.fadeTime)) / Settings.fadeTime;
        if (progress >= 1) then
            -- Fade complete.  If no more messages, shut down the display routine.
            FadeEndTime = nil;
            if (not next(Messages)) then
                Window:SetVisible(false);
                if (currentTime > NextUpdateTime) then
                    Window:SetWantsUpdates(false);
                end
            end
        end
        Window:SetOpacity(math.max(0, math.min(1, 1 - progress)));
    end
end

function ChatReceived(_, args)

    -- Determine which patterns to check for, based on the channel
    local color, loginPattern, logoutPattern, priority;
    if (args.ChatType == Turbine.ChatType.Standard) then
        color = Settings.mainColor;
        loginPattern = FriendLoginPattern;
        logoutPattern = FriendLogoutPattern;
        priority = 1;
    elseif (Settings.showKinmates and (args.ChatType == Turbine.ChatType.Kinship)) then
        color = Settings.kinshipColor;
        loginPattern = KinshipLoginPattern;
        logoutPattern = KinshipLogoutPattern;
        priority = 0;
    else
        return;
    end

    -- Extract name from login or logout message
    local name = string.match(args.Message, loginPattern);
    if ((name == nil) and Settings.showLogouts) then
        name = string.match(args.Message, logoutPattern);
    end

    if (name) then
        if (IsSpecialFriend[name]) then
            color = Settings.specialColor;
        end

        -- If a friend is also a kinmate, we will get a message in both channels, but only want to
        -- display one of them.
        local previousMessage = NewMessages[name];

        -- Record the new message and start the display routine (if necessary).
        if (not previousMessage or (priority >= previousMessage.priority)) then
            NewMessages[name] = { message = args.Message; color = color; priority = priority };
            Window:SetWantsUpdates(true);
        end

        -- Wait for duplicate messages
        if (Settings.showKinmates and not previousMessage) then
            NextUpdateTime = HarvestDelay + Turbine.Engine.GetGameTime();
        else
            NextUpdateTime = 0; -- no need to wait for duplicate messages
        end
    end
end

AddCallback(Turbine.Chat, "Received", ChatReceived);
