-- Default settings
Settings = {
    showLogouts = true; -- Whether to show login and logout messages, or only login messages
    displayTime = 3;    -- How long, in seconds, to display the message before it begins to fade out.
    fadeTime = 1;       -- How long, in seconds, the message takes to fade out.
    mainColor = Turbine.UI.Color.PaleGoldenrod; -- For other colors, see http://www.lotrointerface.com/forums/showthread.php?t=3514
    specialColor = Turbine.UI.Color.DodgerBlue;
    specialFriends = "";
    showKinmates = false; -- Whether to show login and logout messages for non-friend kinmates
    kinshipColor = Turbine.UI.Color.MediumSlateBlue;
};
IsSpecialFriend = {};

OptionsPanel = Turbine.UI.Control();

-- Function for drawing the options panel
function UpdateOptionsPanel()
    local prevContext = L:SetContext("/OptionsPanel");
    local font = Turbine.UI.Lotro.Font.Verdana12;
    local wider = L:GetNumber("IncreasedWidth");
    local secondColumn, thirdColumn = 110 + wider, 176 + wider;
    local left, top, width = 10, 10, 300 + wider;

    if (not OptionsPanel.showLogouts) then
        OptionsPanel.showLogouts = Turbine.UI.Lotro.CheckBox();
        OptionsPanel.showLogouts:SetParent(OptionsPanel);
        OptionsPanel.showLogouts:SetSize(secondColumn + 16 - left, 16);
        OptionsPanel.showLogouts:SetPosition(left, top);
        OptionsPanel.showLogouts:SetFont(font);
        OptionsPanel.showLogouts:SetCheckAlignment(Turbine.UI.ContentAlignment.MiddleRight);
        OptionsPanel.showLogouts:SetText(L:GetText("ShowLogouts"));
        OptionsPanel.showLogouts.CheckedChanged = function(box)
            Settings.showLogouts = box:IsChecked();
        end
    end
    OptionsPanel.showLogouts:SetChecked(Settings.showLogouts);
    top = top + 28;

    if (not OptionsPanel.displayTime) then
        OptionsPanel.displayTimeLabel = Turbine.UI.Label();
        OptionsPanel.displayTimeLabel:SetParent(OptionsPanel);
        OptionsPanel.displayTimeLabel:SetSize(secondColumn, 16);
        OptionsPanel.displayTimeLabel:SetPosition(left, top);
        OptionsPanel.displayTimeLabel:SetFont(font);
        OptionsPanel.displayTimeLabel:SetText(L:GetText("DisplayDuration"));

        OptionsPanel.displayTime = Turbine.UI.Lotro.TextBox();
        OptionsPanel.displayTime:SetParent(OptionsPanel);
        OptionsPanel.displayTime:SetSize(50, 16);
        OptionsPanel.displayTime:SetPosition(secondColumn, top - 2);
        OptionsPanel.displayTime:SetFont(font);
        OptionsPanel.displayTime.TextChanged = function(box)
            Settings.displayTime = tonumber(box:GetText());
        end
    end
    OptionsPanel.displayTime:SetText(tostring(Settings.displayTime));
    top = top + 26;

    if (not OptionsPanel.fadeTime) then
        OptionsPanel.fadeTimeLabel = Turbine.UI.Label();
        OptionsPanel.fadeTimeLabel:SetParent(OptionsPanel);
        OptionsPanel.fadeTimeLabel:SetSize(secondColumn, 16);
        OptionsPanel.fadeTimeLabel:SetPosition(left, top);
        OptionsPanel.fadeTimeLabel:SetFont(font);
        OptionsPanel.fadeTimeLabel:SetText(L:GetText("FadeDuration"));

        OptionsPanel.fadeTime = Turbine.UI.Lotro.TextBox();
        OptionsPanel.fadeTime:SetParent(OptionsPanel);
        OptionsPanel.fadeTime:SetSize(50, 16);
        OptionsPanel.fadeTime:SetPosition(secondColumn, top - 2);
        OptionsPanel.fadeTime:SetFont(font);
        OptionsPanel.fadeTime.TextChanged = function(box)
            Settings.fadeTime = tonumber(box:GetText());
        end
    end
    OptionsPanel.fadeTime:SetText(tostring(Settings.fadeTime));
    top = top + 26;

    if (not OptionsPanel.mainColor) then
        OptionsPanel.mainColorLabel = Turbine.UI.Label();
        OptionsPanel.mainColorLabel:SetParent(OptionsPanel);
        OptionsPanel.mainColorLabel:SetSize(secondColumn, 16);
        OptionsPanel.mainColorLabel:SetPosition(left, top);
        OptionsPanel.mainColorLabel:SetFont(font);
        OptionsPanel.mainColorLabel:SetText(L:GetText("MainColor"));

        OptionsPanel.mainColor = Turbine.UI.Control();
        OptionsPanel.mainColor:SetParent(OptionsPanel);
        OptionsPanel.mainColor:SetSize(50, 16);
        OptionsPanel.mainColor:SetPosition(secondColumn, top - 2);
        OptionsPanel.mainColor.MouseClick = function()
            EditColor(Settings.mainColor, function(newColor) Settings.mainColor = newColor; end);
        end
    end
    OptionsPanel.mainColor:SetBackColor(Settings.mainColor);
    top = top + 26;

    if (not OptionsPanel.specialColor) then
        OptionsPanel.specialColorLabel = Turbine.UI.Label();
        OptionsPanel.specialColorLabel:SetParent(OptionsPanel);
        OptionsPanel.specialColorLabel:SetSize(secondColumn, 16);
        OptionsPanel.specialColorLabel:SetPosition(left, top);
        OptionsPanel.specialColorLabel:SetFont(font);
        OptionsPanel.specialColorLabel:SetText(L:GetText("SpecialColor"));

        OptionsPanel.specialColor = Turbine.UI.Control();
        OptionsPanel.specialColor:SetParent(OptionsPanel);
        OptionsPanel.specialColor:SetSize(50, 16);
        OptionsPanel.specialColor:SetPosition(secondColumn, top - 2);
        OptionsPanel.specialColor.MouseClick = function()
            EditColor(Settings.specialColor, function(newColor) Settings.specialColor = newColor; end);
        end
    end
    OptionsPanel.specialColor:SetBackColor(Settings.specialColor);
    top = top + 26;

    if (not OptionsPanel.showKinmates) then
        OptionsPanel.showKinmates = Turbine.UI.Lotro.CheckBox();
        OptionsPanel.showKinmates:SetParent(OptionsPanel);
        OptionsPanel.showKinmates:SetSize(secondColumn + 16 - left, 16);
        OptionsPanel.showKinmates:SetPosition(left, top);
        OptionsPanel.showKinmates:SetFont(font);
        OptionsPanel.showKinmates:SetCheckAlignment(Turbine.UI.ContentAlignment.MiddleRight);
        OptionsPanel.showKinmates:SetText(L:GetText("ShowKinmates"));
        OptionsPanel.showKinmates.CheckedChanged = function(box)
            Settings.showKinmates = box:IsChecked();
            OptionsPanel.kinshipColorLabel:SetVisible(Settings.showKinmates);
            OptionsPanel.kinshipColor:SetVisible(Settings.showKinmates);
        end
    end
    top = top + 28;

    if (not OptionsPanel.kinshipColor) then
        OptionsPanel.kinshipColorLabel = Turbine.UI.Label();
        OptionsPanel.kinshipColorLabel:SetParent(OptionsPanel);
        OptionsPanel.kinshipColorLabel:SetSize(secondColumn, 16);
        OptionsPanel.kinshipColorLabel:SetPosition(left, top);
        OptionsPanel.kinshipColorLabel:SetFont(font);
        OptionsPanel.kinshipColorLabel:SetText(L:GetText("KinshipColor"));

        OptionsPanel.kinshipColor = Turbine.UI.Control();
        OptionsPanel.kinshipColor:SetParent(OptionsPanel);
        OptionsPanel.kinshipColor:SetSize(50, 16);
        OptionsPanel.kinshipColor:SetPosition(secondColumn, top - 2);
        OptionsPanel.kinshipColor.MouseClick = function()
            EditColor(Settings.kinshipColor, function(newColor) Settings.kinshipColor = newColor; end);
        end
    end
    OptionsPanel.showKinmates:SetChecked(Settings.showKinmates);
    OptionsPanel.showKinmates:CheckedChanged();
    OptionsPanel.kinshipColor:SetBackColor(Settings.kinshipColor);
    top = top + 16 - 2;
    local height = top;
    top = 16;

    if (not OptionsPanel.specialFriends) then
        OptionsPanel.specialFriendsLabel = Turbine.UI.Label();
        OptionsPanel.specialFriendsLabel:SetParent(OptionsPanel);
        OptionsPanel.specialFriendsLabel:SetSize(width - thirdColumn, 16);
        OptionsPanel.specialFriendsLabel:SetPosition(thirdColumn, top);
        OptionsPanel.specialFriendsLabel:SetFont(font);
        OptionsPanel.specialFriendsLabel:SetText(L:GetText("SpecialFriendsList"));
        top = top + 20;

        OptionsPanel.specialFriends = Turbine.UI.Lotro.TextBox();
        OptionsPanel.specialFriends:SetParent(OptionsPanel);
        OptionsPanel.specialFriends:SetSize(width - thirdColumn, height - top);
        OptionsPanel.specialFriends:SetPosition(thirdColumn, top);
        OptionsPanel.specialFriends:SetFont(font);
        OptionsPanel.specialFriends.TextChanged = function(box)
            Settings.specialFriends = box:GetText();

            -- For efficiency, we create a hash table for special friend names
            IsSpecialFriend = {};
            for friend in string.gmatch(Settings.specialFriends, "[^%s,;\n]+") do
                IsSpecialFriend[friend] = true;
            end
        end
    end
    OptionsPanel.specialFriends:SetText(Settings.specialFriends);
    top = height + 12;

    OptionsPanel:SetSize(width, height);
    L:SetContext(prevContext);
end

function EditColor(prevColor, changeFunc)
    if (ColorPickerInstance) then
        ColorPickerInstance:Close();
    end

    ColorPickerInstance = Thurallor.UI.ColorPicker(prevColor);
    ColorPickerInstance.Accepted = function()
        local newColor = ColorPickerInstance:GetColor();
        changeFunc(newColor);
        SaveSettings();
        UpdateOptionsPanel();
        ColorPickerInstance = nil;
    end
    ColorPickerInstance.Canceled = function()
        ColorPickerInstance = nil;
    end
end

function SaveSettings()
--Turbine.Shell.WriteLine("Saving...");
    -- Workaround for Turbine localization bug
    local saveData = ExportTable(Settings);
    Turbine.PluginData.Save(Turbine.DataScope.Account, "FriendAlert", saveData, function()
--Turbine.Shell.WriteLine("Save complete.");
    end);
end

function LoadSettings()
    Turbine.PluginData.Load(Turbine.DataScope.Account, "FriendAlert", function(loadedData)
--Turbine.Shell.WriteLine("Loading...");
        if (loadedData) then
            -- Workaround for Turbine localization bug
            loadedData = ImportTable(loadedData);
            DeepTableCopy(loadedData, Settings);
            local function makeColor(settings)
                return Thurallor.Utils.Color(settings.A, settings.R, settings.G, settings.B);
            end
            Settings.mainColor = makeColor(Settings.mainColor);
            Settings.specialColor = makeColor(Settings.specialColor);
            Settings.kinshipColor = makeColor(Settings.kinshipColor);
            UpdateOptionsPanel();
            OptionsPanel.specialFriends:TextChanged();
--Turbine.Shell.WriteLine("Load complete.");
        end
    end);
end

-- Display the default settings
UpdateOptionsPanel();

-- Create the "options" tab in the plugin manager.
plugin.GetOptionsPanel = function()
    return OptionsPanel;
end

Turbine.Plugin.Load = LoadSettings;
Turbine.Plugin.Unload = SaveSettings;
